const WebSocket = require('ws');
const fs = require('fs');
const path = require('path');

const wss = new WebSocket.Server({ port: 3000 });
const connectionStates = new Map(); // 存储每个连接的状态，避免并发问题

// 获取系统物理驱动器列表（仅示例，实际实现需根据操作系统调整）
function getPhysicalDrives() {
  // Windows系统示例（使用WMIC命令）
  if (process.platform === 'win32') {
    return new Promise((resolve) => {
      const { exec } = require('child_process');
      exec('wmic diskdrive get caption,deviceid,size', (error, stdout) => {
        if (error) {
          console.error('Failed to retrieve the list of drives:', error);
          resolve([]);
          return;
        }
        
        const lines = stdout.split('\n').filter(line => line.trim() !== '');
        const headers = lines.shift().trim().split(/\s+/);
        const drives = [];
        
        lines.forEach(line => {
          const values = line.trim().split(/\s+/);
          if (values.length >= 3) {
            drives.push({
              name: values.slice(0, -2).join(' '),
              path: values[values.length - 2],
              size: formatSize(parseInt(values[values.length - 1]))
            });
          }
        });
        
        resolve(drives);
      });
    });
  }
  
  // Linux系统示例（读取/proc/partitions）
  if (process.platform === 'linux') {
    return new Promise((resolve) => {
      fs.readFile('/proc/partitions', 'utf8', (error, data) => {
        if (error) {
          console.error('Failed to retrieve the list of drives:', error);
          resolve([]);
          return;
        }
        
        const lines = data.split('\n').slice(2).filter(line => line.trim() !== '');
        const drives = [];
        
        lines.forEach(line => {
          const parts = line.trim().split(/\s+/);
          if (parts.length >= 4 && !parts[3].includes('part')) {
            drives.push({
              name: parts[3],
              path: `/dev/${parts[3]}`,
              size: formatSize(parseInt(parts[2]) * 1024)
            });
          }
        });
        
        resolve(drives);
      });
    });
  }
  
  // macOS系统示例（使用diskutil命令）
  if (process.platform === 'darwin') {
    return new Promise((resolve) => {
      const { exec } = require('child_process');
      exec('diskutil list', (error, stdout) => {
        if (error) {
          console.error('Failed to retrieve the list of drives:', error);
          resolve([]);
          return;
        }
        
        const lines = stdout.split('\n');
        const drives = [];
        let currentDrive = null;
        
        lines.forEach(line => {
          if (line.startsWith('/dev/')) {
            const parts = line.trim().split(/\s+/);
            if (parts.length >= 3 && parts[1] === 'disk') {
              currentDrive = {
                name: parts.slice(2).join(' '),
                path: parts[0],
                size: 'N/A'
              };
              drives.push(currentDrive);
            }
          } else if (currentDrive && line.includes('Total Size:')) {
            const sizeMatch = line.match(/Total Size:.*?\((.*?)\)/);
            if (sizeMatch && sizeMatch[1]) {
              currentDrive.size = sizeMatch[1];
            }
          }
        });
        
        resolve(drives);
      });
    });
  }
  
  // 不支持的系统
  return Promise.resolve([]);
}

// 格式化文件大小
function formatSize(bytes) {
  if (bytes === 0) return '0 B';
  const sizes = ['B', 'KB', 'MB', 'GB', 'TB'];
  const i = Math.floor(Math.log(bytes) / Math.log(1024));
  return parseFloat((bytes / Math.pow(1024, i)).toFixed(2)) + ' ' + sizes[i];
}

// 读取物理驱动器扇区
function readSector(device, sectorNumber, sectorCount = 1) {
  return new Promise((resolve, reject) => {
    const sectorSize = 512;
    const bufferSize = sectorSize * sectorCount;
    const buffer = Buffer.alloc(bufferSize);
    
    // 打开设备文件
    fs.open(device, 'r', (err, fd) => {
      if (err) {
        console.error('Failed to open device:', err);
        reject(`Unable to open device: ${device}`);
        
        return;
      }
      
      // 计算偏移量
      const offset = sectorNumber * sectorSize;
      
      // 读取扇区数据
      fs.read(fd, buffer, 0, bufferSize, offset, (err, bytesRead) => {
        fs.close(fd, () => {}); // 关闭文件
        
        if (err) {
          console.error('Reading sector failed:', err);
          reject(`Reading sector failed: ${err.message}`);
          return;
        }
        
        if (bytesRead < bufferSize) {
          console.warn(`Insufficient number of bytes read: ${bytesRead}/${bufferSize}`);
        }
        
        // 将数据转换为Base64编码
        const base64Data = buffer.toString('base64');
        // resolve({
        //   sectorNumber,
        //   sectorData: base64Data
        // });
        resolve(buffer)
      });
    });
  });
}

// WebSocket连接处理
wss.on('connection', (ws) => {
  console.log('客户端已连接');
  
  ws.on('message',  async(message) => {
    try {
      const data = JSON.parse(message);
      
      switch (data.type) {
        case 'GET_DEVICES':
          getPhysicalDrives().then(devices => {
            ws.send(JSON.stringify({
              type: 'DEVICE_LIST',
              devices
            }));
          });
          break;
          
        case 'READ_DISK':
          const { devicesfanhui } = data;
          console.log(data)
          let validDevice = null;
           // 按顺序检查每个设备
          for (let i = 1; i < devicesfanhui.length; i++) {
            const device = devicesfanhui[i].path;
            try {

              const sectorData = await readSector(device, 20480000, 1);
              const signature = sectorData.toString('ascii', 0, 7); // 提取Teswell签名
              
              console.log(`检查设备 ${device} 的签名: ${signature}`);
              
              if (signature === 'Teswell') {
                validDevice = device;
                break; // 找到匹配的设备后跳出循环
              }
            } catch (error) {
              console.error(`检查设备 ${device} 失败:`, error);
              // 继续检查下一个设备
            }
          }
          console.log(`找到有效设备: ${validDevice}`);
          const sectorData = await readSector(validDevice, 20480000, 1);
          // const buffer = Buffer.from(sectorData.sectorData, 'base64')
           const buffer = sectorData; // 直接使用 Buffer

           // 获取最后一个字节，判断存储方式
          const storageMode = buffer[511];
          console.log(`检测到存储模式: ${storageMode}`);

           let fileNameSector;
            let fileCount;

            if (storageMode === 1) { // 2TB存储方式
              fileNameSector = buffer.readUInt32LE(36); // 36-39字节为文件名扇区
              fileCount = buffer.readUInt16LE(44);     // 44-45字节为文件总数
              console.log(`2TB存储方式: 文件名扇区=${fileNameSector}, 文件总数=${fileCount}`);
            } else if (storageMode === 2) { // 4TB存储方式
              fileNameSector = buffer.readUInt32LE(64); // 64-67字节为文件名扇区
              fileCount = buffer.readUInt16LE(80);     // 80-81字节为文件总数
              console.log(`4TB存储方式: 文件名扇区=${fileNameSector}, 文件总数=${fileCount}`);
            } else {
              throw new Error(`不支持的存储方式: ${storageMode}`);
            }

            // 计算需要读取的扇区数
            const bytesNeeded = fileCount * 32; // 每个文件32字节
            const sectorsNeeded = Math.ceil(bytesNeeded / 512);
            console.log(`需要读取 ${sectorsNeeded} 个扇区获取文件名信息`);

            // 读取文件名扇区数据
            const fileNameData = await readSector(validDevice, fileNameSector, sectorsNeeded);
            if (ws) {
              ws.send(JSON.stringify({
                type: 'FILE_INFO',
                fileCount,
                fileNameData: fileNameData.toString('base64'),
                fuwu:validDevice,
              }));
            }
          
          break;
          
        case 'STOP_READING':
          // 停止读取（在实际应用中需要实现取消机制）
          break;
        case 'READ_VIDEO_SECTORS':
            // 停止当前正在进行的读取操作（如果有）
            const wsState = connectionStates.get(ws);
            if (wsState && wsState.isReading) {
              wsState.isReading = false; // 标记为停止
              if (wsState.timeoutId) clearTimeout(wsState.timeoutId); // 清除定时器
              console.log('已停止当前读取任务，准备开始新任务');
            }

            // 初始化新的读取任务状态
            const newState = {
              isReading: true,
              timeoutId: null,
              currentSector: 0,
              remainingSectors: 0
            };
            connectionStates.set(ws, newState);

            const {startSector, endSector} = data;
            const device1 = data.device
            const sectorCount = endSector - startSector + 1;
           
            // 定义时间解析函数（从扇区头部8字节解析时间）
            function parseTime(buffer) {
              const year = buffer[2] + 1900;   // 第3字节（索引2）
              const month = buffer[3];         // 第4字节（索引3）
              const day = buffer[4];           // 第5字节（索引4）
              const hour = buffer[5];          // 第6字节（索引5）
              const minute = buffer[6];        // 第7字节（索引6）
              const second = buffer[7];        // 第8字节（索引7）
              return new Date(year, month - 1, day, hour, minute, second);
            }

            // 计算时间差并格式化为HH:MM
            function formatDuration(startTime, endTime) {
               const durationMs = endTime - startTime;
                const totalSeconds = Math.floor(durationMs / 1000);
                
                const hours = Math.floor(totalSeconds / 3600);
                const minutes = Math.floor((totalSeconds % 3600) / 60);
                const seconds = totalSeconds % 60;
                
                return `${hours.toString().padStart(2, '0')}:${minutes.toString().padStart(2, '0')}:${seconds.toString().padStart(2, '0')}`;
            }

            // 先读取第一个64K和最后一个64K数据块
            const firstBlockSector = startSector;
            const lastBlockSector = Math.max(startSector, endSector - 127); // 最后128个扇区
           // 先读取并解析开始扇区和结束扇区的时间戳
            Promise.all([
              readSector(device1, firstBlockSector, 128),  // 读取开始扇区
              readSector(device1, lastBlockSector, 128)     // 读取结束扇区
            ]).then(([startSectorData, endSectorData]) => {
              // 解析时间戳
              const startTime = parseTime(startSectorData);
              const endTime = parseTime(endSectorData);
              const duration = formatDuration(startTime, endTime);
              
              console.log(`开始时间: ${startTime.toLocaleString()}`);
              console.log(`结束时间: ${endTime.toLocaleString()}`);
              console.log(`时长: ${duration}`);
              ws.send(JSON.stringify({
                type: 'h246time',
                duration
              }));
              // 计算剩余需要读取的扇区数
              let remainingSectors = sectorCount ;
              let currentSector = startSector ;

              const readNextBatch = async () => {

                // 检查是否需要停止或连接是否关闭
                if (!newState.isReading || remainingSectors <= 0 || !ws || ws.readyState !== WebSocket.OPEN) {
                  console.log('读取已停止或完成');
                  newState.isReading = false;
                  connectionStates.delete(ws);
                  return;
                }
                
                const batchSize = Math.min(128, remainingSectors); // 64K数据对应128个扇区
                 // 更新状态
                newState.currentSector = currentSector;
                newState.remainingSectors = remainingSectors;

                try{
                    const batchBuffer = await readSector(device1, currentSector, batchSize);
                    if (ws && ws.readyState === WebSocket.OPEN && newState.isReading) {
                      ws.send(JSON.stringify({
                        type: 'SECTOR_h264DATA',
                        sectors: batchBuffer.toString('hex'),
                        isFirstPacket: (currentSector === startSector), // 标记第一个批次
                        
                      }));
                    }

                    remainingSectors -= batchSize;
                    currentSector += batchSize;

                    if (remainingSectors > 0 && ws && ws.readyState === WebSocket.OPEN && newState.isReading) {
                      newState.timeoutId = setTimeout(readNextBatch, 1);
                    } else if (ws && ws.readyState === WebSocket.OPEN) {
                      ws.send(JSON.stringify({ type: 'READ_COMPLETE' }));
                      newState.isReading = false;
                      connectionStates.delete(ws);
                    }
                }catch (error) {
                  console.error('读取扇区失败:', error);
                  if (ws && ws.readyState === WebSocket.OPEN) {
                    ws.send(JSON.stringify({ type: 'ERROR', message: '读取视频数据失败' }));
                  }
                  newState.isReading = false;
                  connectionStates.delete(ws);
                }
               
      
              };
              readNextBatch();
            });
          
          
          
          
          break;
      }
    } catch (error) {
      console.error('处理消息失败:', error);
      if (ws) {
        ws.send(JSON.stringify({
          type: 'ERROR',
          message: '处理请求失败'
        }));
      }
    }
  });
  
  ws.on('close', () => {
    console.log('客户端已断开连接');
  });
});

console.log('WebSocket服务器已启动，端口3000');